/*
 *  Copyright (C) 2002 Tomasz Kojm <zolw@konarski.edu.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "readdb.h"
#include "others.h"
#include "strings.h"
#include "shared.h"

int parse_add(struct signature **sigtab, const char *virname, const char *hexstr)
{
	struct signature *newsign;

    /* decode hexstring and prepare new node */

    newsign = (struct signature *) mmalloc(sizeof(struct signature));
    newsign->pattlen = strlen(hexstr) / 2;
    newsign->flag = 0;

    if((newsign->pattern = hex2str(hexstr)) == NULL) {
	free(newsign);
	return 1;
    }

    newsign->virname = mcalloc(strlen(virname) + 1, sizeof(char));
    memcpy(newsign->virname, virname, strlen(virname));

    newsign->next = sigtab[(unsigned char)(newsign->pattern[0])]->next;
    sigtab[(unsigned char)(newsign->pattern[0])]->next = newsign;

    return 0;
}

/* this functions returns a pointer to the list with parsed signatures */

struct signature **readdb(const char *filename)
{
	FILE *fd;
	char buffer[BUFFSIZE], *pt, *start;
	int line = 0, i;
	struct signature **sigtab;

    if((fd = fopen(filename, "rb")) == NULL) {
	mprintf("!Can't open file %s\n", filename);
	return NULL;
    }

    /* prepare signature table */
    sigtab = mcalloc(ALL, sizeof(struct signature *));

    for(i = 0; i <= ALL; i++) {
	sigtab[i] = malloc(sizeof(struct signature));
	sigtab[i]->next = NULL;
    }

    while(fgets(buffer, BUFFSIZE, fd)) {
	line++;
	chomp(buffer);

	pt = strchr(buffer, '=');
	if(!pt) {
	    mprintf("!Malformed pattern line %d.\n", line);
	    //FIXME: free_list()
	    return NULL;
	}

	start = buffer;
	*pt++ = 0;

	if(*pt == '=') continue;

	claminfo.signs++;
	if(parse_add(sigtab, start, pt)) return NULL;
    }

    fclose(fd);
#ifdef PRINT_SIG
    print_signature_list(sigtab);
    exit(0);
#endif
    return sigtab;
}

#ifdef PRINT_SIG
void print_signature_list(struct signature **sigtab)
{
	int i, j;
	struct signature *list;
	FILE *fd;

    fd = fopen("DecodeDB", "wb");

    for(i = 0; i < ALL; i++) {
	list = sigtab[i]->next;

	while(list) {
	    mprintf("\nVIRUS NAME: %s\nSIGNATURE: ", list->virname);
	    fwrite(list->pattern, 1, list->pattlen, fd);
	    fprintf(fd, "\n");
	    list = list->next;
	}
    }

    fclose(fd);
}
#endif
